'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _babelAddFlowComments = require('babel-add-flow-comments');

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _slash = require('slash');

var _slash2 = _interopRequireDefault(_slash);

var _uppercamelcase = require('uppercamelcase');

var _uppercamelcase2 = _interopRequireDefault(_uppercamelcase);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var babylonOpts = { sourceType: 'module', plugins: ['flow'] };

var wrapTemp = function wrapTemp(tmpl) {
  return (0, _babelTemplate2.default)(tmpl, babylonOpts);
};

var createUnion = function createUnion(union) {
  return wrapTemp('export type Action = UNION')({
    UNION: t.unionTypeAnnotation(union)
  });
};

function trimExtension(path) {
  var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';

  return (0, _path.extname)(path) === ext ? path.replace(ext, '') : path;
}

function getImportPath(from, to) {
  var relativePath = (0, _slash2.default)((0, _path.relative)((0, _path.dirname)(from), to));
  var fomattedPath = trimExtension(relativePath);
  if (!/^\.\.?/.test(fomattedPath)) {
    return './' + fomattedPath;
  }
  return fomattedPath;
}

function createActionName(path) {
  var parentPath = (0, _path.normalize)((0, _path.dirname)(path)).split('/');
  return (0, _uppercamelcase2.default)(parentPath[parentPath.length - 1]) + 'Action';
}

exports.default = function () {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-action-root',
    visitor: {
      Program: {
        exit: function exit(programPath, state) {
          var file = state.file;

          (0, _babelAddFlowComments.removeFlowComment)(file.ast.comments);
          var _state$opts = state.opts,
              input = _state$opts.input,
              output = _state$opts.output;

          var globOptions = Object.assign({ absolute: true }, state.opts.globOptions);

          if (!input) {
            throw new Error('require input option');
          }

          if (!output) {
            throw new Error('require output option');
          }

          var files = _globby2.default.sync(input, globOptions);

          var imports = files.map(function (f) {
            return {
              source: getImportPath(output, f),
              name: createActionName(f)
            };
          }).map(function (_ref) {
            var name = _ref.name,
                source = _ref.source;

            var im = t.importDeclaration([t.importSpecifier(t.identifier(name), t.identifier('Action'))], t.stringLiteral(source));
            // $FlowFixMe
            im.importKind = 'type';
            return im;
          });

          var union = files.map(createActionName).map(function (name) {
            return t.genericTypeAnnotation(t.identifier(name));
          });

          var action = createUnion(union);

          programPath.node.body = [].concat(_toConsumableArray(imports), [t.noop(), action]);

          (0, _babelAddFlowComments.addFlowComment)(programPath);
        }
      }
    }
  };
};